/*
 * @(#)ImageSelection.java
 *
 */

import java.awt.*;
import java.awt.datatransfer.*;

/**
 *
 * <p>Fichier : ImageSelection.java</p>
 * <p>Description :</p>
 * Cette classe est un emballage (wrapper) pour le transfert de donnees des
 * objets de type image (dans le presse-papiers par exemple)
 * <p>
 * Les objets qui peuvent etre transferes via un presse-papiers doivent
 * implementer l'interface Transferable.
 * La classe DataFlavor decrit le type de donnees qui peuvent etre placees
 * dans un presse-papier.
 * Le SDK fournit la constante Dataflavor.imageFlavor qui effectue le lourd
 * travail permettant de convertir des images Java et des images natives
 * du presse-papiers. Toutefois, curieusement, il ne fournit pas de classe
 * d'enveloppe (wrapper) necessaire pour placer les images dans le
 * presse-papiers (contrairement a StringSelection pour les chaines de
 * caracteres).
 * <p>Copyright : Copyright (c) 04/2004</p>
 * @author A. MARI
 * @version 1.0
 */
public class ImageSelection implements Transferable {

  // Image a placer dans le presse-papiers
  private Image image;

  /**
   * Construit la classe enveloppe (wrapper) qui va servir a placer 'image'
   * dans le presse-papiers
   * @param image image a placer dans le presse-papiers.
   */
  public ImageSelection (Image image) {
    this.image = image;
  }

  /**
   * Renvoie la donnee formatee selon le type specifie. Declenche une
   * UnsupportedFlavorException si le type specifie n'est pas supporte
   * @param flavor type de donnee specifie
   * @return donnee formattee selon le type specifie (DataFlavor.imageFlavor)
   * @throws UnsupportedFlavorException lancee si le type specifie n'est pas
   * supporte
   */
  public Object getTransferData(DataFlavor flavor)
      throws UnsupportedFlavorException {
    if (flavor.equals(DataFlavor.imageFlavor)) {
      return image;
    }
    else {
      throw new UnsupportedFlavorException(flavor);
    }
  }

  /**
   * Renvoie le type standard pour les images
   * @return type DataFlavor.imageFlavor pour les images
   */
  public DataFlavor[] getTransferDataFlavors() {
    return new DataFlavor[] {DataFlavor.imageFlavor};
  }

  /**
   * Renvoie true si le type 'flavor' fait partie du type de donnees
   * DataFlavor.imageFlavor et false sinon.
   * @param flavor type de donnees a tester pour savoir s'il fait partie du
   * type de donnees image (DataFlavor.imageFlavor)
   * @return true si le type 'flavor' fait partie du type
   * DataFlavor.imageFlavor et false sinon
   */
  public boolean isDataFlavorSupported(DataFlavor flavor) {
    return flavor.equals(DataFlavor.imageFlavor);
  }
}
